/*******************************************************************************
    Copyright (c) 2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef __UVM8_RANGE_GROUP_H__
#define __UVM8_RANGE_GROUP_H__

#include "nvtypes.h"
#include "uvm8_range_tree.h"

typedef struct uvm_range_group_struct
{
    // Group ID
    NvU64 id;

    // Does this group allow migration?
    // This is not protected by a lock, which is okay since it is valid
    // for a range group to be set as migratable while a migration of its
    // VA ranges to their preferred location has not been completed yet.
    // Using atomic_t rather than bool here to emphasize the fact that this
    // field is special
    atomic_t allow_migration;

    // VA ranges in this group
    struct list_head va_ranges;
} uvm_range_group_t;

static inline bool uvm_range_group_migratable(uvm_range_group_t *range_group)
{
    // A NULL range group is always migratable
    return !range_group || atomic_read(&range_group->allow_migration);
}

NV_STATUS uvm_range_group_init(void);
void uvm_range_group_exit(void);
void uvm_range_group_radix_tree_destroy(struct radix_tree_root *tree);

#endif // __UVM8_RANGE_GROUP_H__
