/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef __UVM_FULL_CTX_MGMT_H__
#define __UVM_FULL_CTX_MGMT_H__

#include "uvm_linux.h"
#include "uvmtypes.h"

NV_STATUS uvmfull_ctx_mgmt_slab_init(void);
void uvmfull_ctx_mgmt_slab_destroy(void);

NV_STATUS uvmfull_vaspace_init(struct inode *inode, struct file *fp);
NV_STATUS uvmfull_vaspace_destroy(struct inode *inode, struct file *fp);

NV_STATUS uvmfull_gpu_vaspace_activate(struct file* filp, UvmGpuUuid *uuid,
                                       NvHandle hClient, NvHandle hVaspace);
NV_STATUS uvmfull_gpu_vaspace_deactivate(struct file* filp, UvmGpuUuid *uuid);
NV_STATUS uvmfull_register_gpu_vaspace(struct file* filp, UvmGpuUuid *uuid,
                                       NvHandle hClient, NvHandle hVaspace);
NV_STATUS uvmfull_unregister_gpu_vaspace(struct file* filp, UvmGpuUuid *uuid);

NV_STATUS uvmfull_gpu_vaspace_add_channel(struct file* filp,
                                          UvmGpuUuid *uuid,
                                          NvHandle hClient,
                                          NvHandle hChannel,
                                          NvU64 base,
                                          NvLength length);

NV_STATUS uvmfull_gpu_vaspace_remove_channel(struct file* filp,
                                             UvmGpuUuid *uuid,
                                             NvHandle hChannel);

NvU64 uvmfull_instptr_key(NvU64 instptr, UvmFaultInstLoc loc);
UvmVaspace *uvmfull_get_vaspace_from_instptr(UvmGpuId gpuid, NvU64 instptr,
                                             UvmFaultInstLoc loc);

NvBool uvmfull_gpu_vaspace_active(UvmVaspace *vaspace, UvmGpuId gpuId);

NV_STATUS uvmfull_enable_peer_on_vaspace(UvmVaspace *vaspace, UvmGpuId gpuId1,
                                         UvmGpuId gpuId2);
NV_STATUS uvmfull_disable_peer_on_vaspace(UvmVaspace *vaspace, UvmGpuId gpuId1,
                                          UvmGpuId gpuId2);
NvBool uvmfull_vaspace_is_peeraccess_valid(UvmVaspace *vaspace, UvmGpuId gpuId1,
                                           UvmGpuId gpuId2);
NV_STATUS uvmfull_vaspace_peermap_destroy(struct file *filp);

NV_STATUS uvmfull_map_rm_allocation(struct file* filp, NvU64 base,
                            NvU64 length, UvmGpuMappingAttributes *perGpuAttribs,
                            NvLength attribCount, NvHandle hClientFromRM,
                            NvHandle hMemoryFromRm);

#endif  // __UVM_FULL_CTX_MGMT_H__

