/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef __UVM_FULL_DEVICE_MGMT_H__
#define __UVM_FULL_DEVICE_MGMT_H__

#include "uvm_full.h"
#include <linux/workqueue.h>

// <unreviewed_code>

struct UvmPendingFault_tag
{
    struct work_struct work;
    UvmGpuUuid uuid;
};

UvmGpu* uvmfull_gpuid_to_gpu(UvmGpuId id);

UvmGlobalState* uvmfull_get_global(void);

//
// Compares the two gpu uuids u1 and u2 and returns an integer 
// less than, equal to, or greater than zero if u1, respectively,
// is less than, matches, or is greater than u2.
//
NvS32 uvm_gpu_uuid_cmp(UvmGpuUuid *u1, UvmGpuUuid *u2);

NV_STATUS uvmfull_device_init(void);
NV_STATUS uvmfull_device_destroy(void);
NV_STATUS uvmfull_start_gpu(UvmGpuUuid *pGpuUuid);
NV_STATUS uvmfull_stop_gpu(UvmGpuUuid *pGpuUuid);
NV_STATUS uvmfull_stop_gpuid(UvmGpuId id);
NV_STATUS uvmfull_isr_top_half_impl(UvmGpuUuid* pUuid, NvBool bSynchronous,
                                    NvBool *pbBufferLockAcquired);

UvmGpu* uvmfull_gpuid_to_gpu(UvmGpuId id);
UvmGpu* uvmfull_gpuid_to_active_gpu(UvmGpuId id);
UvmGpu* uvmfull_uuid_to_gpu(UvmGpuUuid *pGpuUuid);
UvmGpu* uvmfull_uuid_to_active_gpu(UvmGpuUuid *uuid);
UvmGpu* uvmfull_intr_uuid_to_active_gpu(UvmGpuUuid *uuid);
UvmGpu* uvmfull_intr_gpuid_to_active_gpu(UvmGpuId id);
UvmGpu* uvmfull_peerid_to_gpu(UvmGpuId fromGpuId, NvU32 peerId);

NV_STATUS uvmfull_enable_peer_access(struct file* filp,
                                     UvmGpuUuid *gpuUuid1,
                                     UvmGpuUuid *gpuUuid2);
NV_STATUS uvmfull_disable_peer_access(struct file* filp,
                                      UvmGpuUuid *gpuUuid1,
                                      UvmGpuUuid *gpuUuid2);

NV_STATUS uvmfull_init_gpu_faultbuffer(UvmGpu *pGpu);
NV_STATUS uvmfull_destroy_gpu_faultbuffer(UvmGpu *pGpu);
NvBool uvmfull_is_gpu_fault_capable(UvmGpuId gpuId);

void uvmfull_init_global_state(void);
NV_STATUS uvmfull_device_mgmt_slab_init(void);
void uvmfull_device_mgmt_slab_destroy(void);

// </unreviewed_code>

NV_STATUS uvmfull_isr_top_half(UvmGpuUuid *pGpuUuid);

#endif
