/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/
#ifndef __UVM_FULL_FAULT_HANDLER_H__
#define __UVM_FULL_FAULT_HANDLER_H__

#define UVM_PREFETCH_THROTTLE_THRESHOLD       256
#define UVM_PREFETCH_PENALTY_WEIGHT           2
#define NV_UVM_GPU_DEFAULT_FAULT_IN_SIZE             PAGE_SIZE_64K


typedef struct
{
    UvmFaultBufferEntry entry;
    NvU32 get;              // The get index at which this was found
    NvU32 bValid           : 1; //! Coalesce logic can make this invalid
    NvU32 bInCancelWindow  : 1; //! Should not be replayed
    NvU32 bCancelWindowEnd : 1; //! Cancel should be triggered
    NvU32 bFatalFault      : 1; //! A cancel would be required for this fault.
    NvU32 bPrefetch        : 1; //! Prefetch access. No need to resolve.
    NvU32 bReplayed        : 1; //! A replay was pushed for this fault item
} UvmFaultItem;

NV_STATUS uvmfull_handle_gpu_single_fault
(
    UvmGpu                  *pGpu,
    UvmVaspace              *pVaspace,
    UvmFaultItem            *pFaultItem
);
NV_STATUS uvmfull_handle_gpu_faults(UvmGpuUuid *pGpuUuid);
NV_STATUS uvmfull_allocate_phys_sys_mem
(
    NvU32                   numPages,
    NvU64                   **ppPhysPageList
);
NV_STATUS uvmfull_allocate_phys_vid_mem
(
    UvmGpu                  *pGpu,
    NvU64                   size,
    NvU64                   *pPfn
);
void uvmfull_free_phys_vid_mem
(
    UvmGpu                  *pGpu,
    NvU64                   pfn
);
NV_STATUS uvmfull_handle_gpu_pde_fault
(
    UvmGpu                  *pGpu,
    Uvm2MbVaDesc            *pVaDesc,
    UvmGpuPde               *pGpuPde,
    UvmFaultItem            *pFaultItem
);
NV_STATUS uvmfull_handle_gpu_pte_fault
(
    UvmGpu                  *pGpu,
    Uvm2MbVaDesc            *pVaDesc,
    UvmGpuPde               *pGpuPde,
    UvmFaultItem            *pFaultItem,
    UvmPbOps                *pbOps
);
NV_STATUS uvmfull_alloc_backing_gpu_pages
(
    UvmGpu                  *pGpu,
    UvmGpuPde               *pGpuPde,
    NvU32                   pgSize,
    NvU32                   numPages
);
void uvmfull_free_backing_gpu_pages
(
    UvmGpu                  *pGpu,
    UvmGpuPde               *pGpuPde
);
NV_STATUS uvmfull_handle_gpu_write_fault
(
    UvmGpu                  *pGpu,
    Uvm2MbVaDesc            *pVaDesc,
    UvmGpuPde               *pGpuPde,
    UvmFaultItem            *pFaultItem
);

NV_STATUS uvmfull_handle_gpu_atomic_fault
(
    UvmGpu                  *pGpu,
    Uvm2MbVaDesc            *pVaDesc,
    UvmGpuPde               *pGpuPde,
    UvmFaultItem            *pFaultItem
);

NV_STATUS uvmfull_flush_faultbuffer_for_channel
(
    UvmClientChannel        *channel
);

int uvmfull_cpu_fault_handler(struct vm_area_struct *vma, NvU64 faultingAddress,
                              struct page **ppage, unsigned vmfFlags);
NV_STATUS uvmfull_init_fault_cancel_cache(void);
void uvmfull_destroy_fault_cancel_cache(void);

NvBool uvmfull_check_page_merge(Uvm2MbVaDesc *pUvm2MbVaDesc, UvmGpuPde *pGpuPde,
                  NvU64 vaddr, UvmGpuPte *pPte, NvU32 curPgSize,
                  NvU32 mergePageSize, NvU64* startPfn);
void uvmfull_check_page_split(Uvm2MbVaDesc *pUvm2MbVaDesc, NvU64 vaddr,
        NvU32 curPageSize, NvU32 *pSuggestedPgSize);
NV_STATUS uvmfull_unmap_copy_cpu_to_gpu(Uvm2MbVaDesc *pUvm2MbVaDesc,
                                        UvmGpuPde *pGpuPde, UvmPushbuffer *pb,
                                        NvU32 offset, NvU32 cpuPages);

#endif   // __UVM_FULL_FAULT_HANDLER_H__
