/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

//
// uvm_full_fidentity_map.h
//
// This file contains structures and function declarations to create and destroy
// identity mappings to support PASCAL P2P.

#ifndef _UVM_FULL_IDENTITY_MAP_H_
#define _UVM_FULL_IDENTITY_MAP_H_

#include "uvmtypes.h"
#include "uvm_full.h"
#include "uvm_full_types.h"
#include "uvm_channel_mgmt.h"

// These reservation is PASCAL specific where
// rootPD has 4 entires.
// Out of which, rootPDE[0] i.e. [0 128TB) is reserved for RM
// and rest 128TB+ are managed by UVM.
#define IDENTITY_MAPPING_VA_BASE 0x800000000000ULL

//MAX FB 32 GB
#define PASCAL_MAX_FB 0x800000000ULL

#define GET_PEERMAP_VA_FROM_PA(peerId, PA) ((PA) + IDENTITY_MAPPING_VA_BASE + ((peerId) * PASCAL_MAX_FB))

/******************************************************************************
    uvmfull_create_identity_map
        Sets up identity mapping for a given GPU ID

    Arguments:
        pGpu: (INPUT/OUTPUT)
            Gpu info. Tracks vidmem PA using a list.

    Returns:
        NV_OK: if the operation succeeds.
        NV_ERR_NO_MEMORY: if there isnt enough memory to setup pageTables.
        NV_ERR_IN_USE: if there isnt enough pushBuffer mempry.
        NV_ERR_INVALID_ARGUMENT: In case of NULL args.
*/

NV_STATUS uvmfull_create_identity_map(UvmGpu *pGpu);

/******************************************************************************
    uvmfull_destroy_identity_map
        Deletes identity mapping for a given GPU ID

    Arguments:
        pGpu: (INPUT)
            Gpu info. Tracks vidmem PA using a list.

    Returns:
        NV_OK: if the operation succeeds
        NV_ERR_INVALID_ARGUMENT: In case of NULL args.
*/

NV_STATUS uvmfull_destroy_identity_map(UvmGpu *pGpu);

/******************************************************************************
    Setup cache
*/
NV_STATUS uvmfull_identity_map_cache_init(void);

/******************************************************************************
    Destroy cache
*/
void uvmfull_identity_map_cache_destroy(void);

#endif // _UVM_FULL_IDENTITY_MAP_H_

