/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef _UVM_FULL_PAGETBL_MGMT_H_
#define _UVM_FULL_PAGETBL_MGMT_H_

#include "uvm_linux.h"
#include "uvmtypes.h"
#include "uvm_full.h"

struct UvmGpuPb_tag
{
    UvmGpuId        gpuId;
    UvmPushbuffer   *pb;
    NvBool          bSubmitted;
};

struct UvmPbOps_tag
{
    NvU8          nGpus;
    UvmGpuPb      *gpuPb;   // array of gpuPbs
    UvmTracker    *t;
};

// GPU PDE list iterator
#define desc_for_each_gpu_pde(pde, desc2M)                                  \
    list_for_each_entry(pde, &((desc2M)->gpuPDEs), pdeNode)

#define desc_for_each_gpu_pde_safe(pde, tempPde, desc2M)                    \
    list_for_each_entry_safe(pde, tempPde, &((desc2M)->gpuPDEs), pdeNode)

NV_STATUS uvmfull_pagetbl_mgmt_slab_init(void);
void uvmfull_pagetbl_mgmt_slab_destroy(void);
NV_STATUS uvmfull_alloc_va_range(UvmVaspace *pVaspace,
                                 NvU64      addr,
                                 NvU64      length,
                                 UvmPteHintsParams *pteHints);
NvU32 uvmfull_reclaim_va_desc(Uvm2MbVaDesc *vaDesc, UvmVatL0Entry **parentLevel,
                              NvU64 vaddr, NvU32 n4k);
void uvmfull_delete_va_desc(Uvm2MbVaDesc *pUvm2MVaDesc);
UvmGpuPde *uvmfull_get_gpupde_in_vadesc(Uvm2MbVaDesc *pUvm2MbVaDesc, 
                                            UvmGpuId gpuId);
Uvm2MbVaDesc* uvmfull_alloc_va_desc(UvmVaspace *pVaspace,
                                  NvU64 addr);
NV_STATUS uvmfull_alloc_va_range(UvmVaspace *pVaspace,
                                 NvU64 addr, 
                                 NvU64 length,
                                 UvmPteHintsParams *pPteHints);

UvmGpuPde* uvmfull_alloc_gpupde(Uvm2MbVaDesc *pUvm2MbVaDesc, UvmGpuId gpuId);

void uvmfull_destroy_gpupde(Uvm2MbVaDesc *pUvm2MbVaDesc, UvmGpuId gpuId);

void uvmfull_unmap_gpu_pde_pages(UvmVaspace *vaspace, UvmGpuId gpuId,
                                 UvmPbOps *pbOps, Uvm2MbVaDesc *vaDesc,
                                 UvmGpuPde *gpuPde, NvU64 vaddr,
                                 NvU32 numPages, NvBool skipTlbInvalidate);

NV_STATUS uvmfull_unmap_on_gpus(UvmVaspace *vaspace, NvU64 base,
                                NvU64 length, NvU8 *gpuids, NvU32 numGpus,
                                NvBool skipTlbInvalidate);
NvU32 create_pte_template(UvmGpuVaspace *gpuVaspace, NvU8 *pteTemplate,
                        NvU8 templateSize,
                        NvBool bValid, NvBool bVol, UvmMemType aperture,
                        NvBool bAtomic, NvBool bReadOnly);
NV_STATUS uvmfull_map_gpus_unmanaged_va(UvmVaspace *vaspace, NvU64 base,
                                    NvU8 *gpuids,
                                    UvmGpuSurfaceMappingInfo *surfMapInfo,
                                    UvmGpuMappingAttributes *perGpuAttribs,
                                    NvLength attribCount);

NV_STATUS uvmfull_update_gpu_pte(UvmVaspace *pVaspace,
                                 UvmGpuId   gpuid, 
                                 UvmGpuPte  *pPte, 
                                 NvU64      vaddr, 
                                 NvU32      pteUpdateHwFlags);

NV_STATUS uvmfull_update_gpupde_ptes(
                Uvm2MbVaDesc            *pUvm2MbVaDesc,
                UvmGpuId                gpuid,
                UvmGpuPde               *pGpuPde,
                NvU64                   addr,
                NvU64                   numPages,
                NvU32                   pgSize,
                UvmGpuPteUpdateParams   *pPteParams,
                UvmGpuPfn               *pfnList,
                UvmPbOps                *pbOps,
                NvU8                    *pteTemplate,
                NvU32                   entrySize,
                NvBool                  skipTlbInvalidate);

UvmGpuPte* uvmfull_get_gpupde_current_pte(UvmGpuPde *pGpuPde, 
                                          NvU64     addr, 
                                          NvU32     *pPageSize);
UvmGpuPte* uvmfull_get_gpupde_pte(UvmGpuPde *pGpuPde, NvU32 pgSize, NvU64 addr);

NV_STATUS uvmfull_merge_gpupde_pte(UvmVaspace *pVaspace, UvmGpuId gpuid,
        Uvm2MbVaDesc *pUvm2MbVaDesc, UvmGpuPde *pGpuPde, NvU64 addr,
        NvU32 newPgSize, UvmPbOps *pbOps);

NV_STATUS uvmfull_split_gpupde_pte(UvmVaspace *pVaspace, UvmGpuId gpuid,
        Uvm2MbVaDesc *pUvm2MbVaDesc, UvmGpuPde *pGpuPde, NvU64 addr,
        NvU32 newPgSize, NvU32 splitFlags, UvmPbOps *pbOps);

NvLength uvmfull_encode_gpu_pte(NvU8 *bufferStart, NvU8 *bufferEnd,
                                GMMU_FMT *gmmuFmt, UvmGpuPte *pPte,
                                NvU8 *template, NvU32 entrySize);
NV_STATUS uvmfull_get_pagetbl_memory(NvU64 size, NvU64 *pfn,
                                        UvmMemType aperture, UvmGpuId gpuId);

void uvmfull_free_pagetbl_memory(NvU64 pfn, NvU64 size, UvmMemType aperture,
                                UvmGpuId gpuId);

NV_STATUS uvmfull_activate_gpupte(Uvm2MbVaDesc *vaDesc, UvmGpuPde* gpuPde,
                              NvBool bLinkBig, NvBool bLinkSmall, UvmGpuId gpuId,
                              UvmPushbuffer *pb);

NvLength uvmfull_encode_gpu_pde(NvU8 *bufferStart, NvU8 *bufferEnd,
                             GMMU_FMT *gmmuFmt,
                             NvU32 level, NvU64 levelLoBit,
                             NvU64 physAddrSub0, UvmMemType apertureSub0,
                             NvU64 physAddrSub1, UvmMemType apertureSub1);
void uvmfull_update_gpu_pde_entry(UvmGpuVaspace *gpuVaspace, UvmPushbuffer *pb,
            NvU32 parentLevel, NvU64 parentPhyAddr, UvmMemType parentAperture,
            NvU32 entryId, NvU8 parentLoBit,
            NvU64 pointeeAddrSub0, UvmMemType pointeeApertureSub0,
            NvU64 pointeeAddrSub1, UvmMemType pointeeApertureSub1,
            NvBool skipTlbInvalidate);

NV_STATUS uvmfull_vadesc_update_gpupde_info(Uvm2MbVaDesc *vaDesc,
                                            UvmGpuId gpuId,
                                            UvmVatLevelGpuHeader *gpuHeader,
                                            NvU32 entryId);

NV_STATUS uvmfull_is_gpupte_linked(UvmGpuPde* gpuPde, NvBool *bigLinked,
                                NvBool *smallLinked);

UvmPbOps* uvm_pbops_start(UvmVaspace *vaspace);
UvmPbOps* uvm_pbops_start_on_gpu(UvmGpuId gpuId);
void      uvm_pbops_end(UvmPbOps *pbOps);

static inline UvmPushbuffer* uvm_pbops_gpu_pb(UvmPbOps *pbOps, UvmGpuId id)
{
    NvU32 j;
    if (!pbOps->nGpus || !pbOps->gpuPb)
        return NULL;

    for (j=0; j<pbOps->nGpus; j++)
    {
        if (pbOps->gpuPb[j].gpuId == id)
        {
            return pbOps->gpuPb[j].pb;
        }
    }
    return NULL;
}

NV_STATUS uvm_pbops_submit(UvmPbOps *pbOps, UvmGpuId id,
                            UvmPushbuffer *pb, UvmTracker *tracker);
NV_STATUS uvm_pbops_submit_and_release(UvmPbOps *pbOps, NvU64 gpuMask,
                                       UvmTracker *tracker);
void uvm_pbops_reserve_acquire_space(UvmPbOps *pbOps, UvmTracker *tracker);

void uvm_push_gpu_to_cpu_copy(UvmPushbuffer *pushbuffer, NvU64 src, NvU64 dst,
                              NvU64 size, NvU32 flags);

void uvm_push_cpu_to_gpu_copy(UvmPushbuffer *pushbuffer, NvU64 src, NvU64 dst,
                              NvU64 size, NvU32 flags);

void uvm_push_copy(UvmPushbuffer *pushbuffer, NvU64 src, NvU64 dst,
                              NvU64 size, NvU32 flags, UvmMemType srcAperture,
                              UvmMemType dstAperture);

//
// This function does memset based copy from a given
// src buffer to a given destination.
//
NV_STATUS uvm_push_memset(UvmPushbuffer *pb,
                              NvU8 *srcStart,
                              NvU64 copySize,
                              NvU64 dst,
                              NvU32 dstAperture,
                              NvU32 dstCopyFlags);

// Copy a PTE using memset.
NV_STATUS
uvm_copy_pte_using_memset(UvmGpuVaspace *gpuVaspace,
                        UvmGpuPde *pGpuPde,
                        UvmGpuPte* pPte,
                        NvU64 addr,
                        NvU32 pgSize,
                        UvmPushbuffer *pb);

void uvm_trigger_replay_priv(UvmGpuId gpuId,
                        UvmFaultBufferEntry *pFault,
                        UvmTlbInvalidateReplayType replayType,
                        UvmMemType aperture, NvU32 flags);

void uvm_push_invalidate(UvmGpuId gpuId,
                         UvmPushbuffer *pushbuffer,
                         UvmTlbInvalidateVaParams invalidateParams,
                         UvmMemType aperture, NvU64 targetPdb,
                         UvmTlbInvalidateMembarType membarType);

void uvm_push_replay(UvmGpuId gpuId,
                     UvmPushbuffer *pushbuffer,
                     NvBool bReplayStartAckAll,
                     UvmTlbInvalidateVaParams invalidateParams,
                     UvmMemType aperture, NvU64 targetPdb);

void uvm_push_cancel(UvmGpuId gpuId,
                     UvmPushbuffer *pushbuffer,
                     NvU32 gpcId,
                     NvU32 clientId,
                     NvBool bGlobalCancel,
                     UvmTlbInvalidateVaParams invalidateParams,
                     UvmMemType aperture, NvU64 targetPdb);

void uvm_push_memops_raw(UvmGpuId gpuId,
                         UvmPushbuffer *pushbuffer,
                         NvU32 gpcId,
                         NvU32 clientId,
                         UvmTlbInvalidateReplayType replayType,
                         UvmTlbInvalidateAckType ackType,
                         UvmTlbInvalidateVaParams invalidateParams,
                         UvmTlbInvalidateMembarType membar,
                         UvmMemType aperture,
                         NvBool disableGpcInvalidate,
                         NvU64 targetPdb);

void uvm_push_membar(UvmGpuId gpuId,
                     UvmPushbuffer *pushbuffer,
                     UvmTlbInvalidateMembarType membarType);

void uvm_push_host_wfi(UvmGpuId gpuId,
                     UvmPushbuffer *pushbuffer);

void uvm_push_memzero_vidmem(UvmGpuId gpuId, UvmPushbuffer *pb,
                        NvU64 pagePa, NvU64 size);

GMMU_APERTURE uvm_memtype_to_gmmu_aperture(UvmMemType aperture);

NvU32 uvm_memtype_to_memset_dst_flags(UvmMemType aperture);

NvU32 uvm_memtype_to_copy_dst_flags(UvmMemType aperture);

NV_STATUS uvmfull_access_memory(UvmVaspace *vaspace,
                                NvU64 userCopyBuffer,
                                NvU64 base,
                                NvU64 length,
                                UvmDebugAccessType accessType,
                                NvU64 *bitmask,
                                NvBool *isBitmaskSet);


void uvmfull_update_pte_hints(NvU64 addr, NvU64 numPages, NvU32 pgSize,
                              UvmPteHintsParams *pteHintsParams, 
                              Uvm2MbVaDesc *pVaDesc2M);
#endif // _UVM_FULL_PAGETBL_MGMT_H_
