/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef __UVM_FULL_TEST__
#define __UVM_FULL_TEST__

#include "uvmtypes.h"

#define UVM_TEST

#include "uvmfull_test_ioctl.h"
#include "uvm_full.h"

long uvmfull_test_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);

// Commit/DeCommit
/* UvmTestRegionCommit :
 *   Tells the UVM driver that this region in VA is going to be used in the test
 *   The function does two things:
 *   a. mmap ( which should call .map in the sturct registered in uvm*_init
 *   b. Call UVM_TEST_REGION_COMMIT ioctl for the VA range for passing more args
 */
NV_STATUS uvm_test_region_commit(
             UVM_TEST_REGION_COMMIT_PARAMS *pParams, struct file *filp);

//Set state
/* uvm_test_set_va_state: ioctl to set initial state per GPU
 * 1. Alloc GPU backing by calling rm function
 * 2. Mark it 'invalid' so as to fault if GPU hits it.
 */
NV_STATUS uvm_test_set_va_state(
             UVM_TEST_SET_VA_STATE_PARAMS *stateParams, struct file *filp);

//Set fault policy
NV_STATUS uvm_test_set_fault_policy(
            UVM_TEST_SET_FAULT_POLICY_PARAMS* policyParams, struct file *filep);

NV_STATUS uvm_test_register_device(
            UVM_TEST_REGISTER_DEVICE_PARAMS* params, struct file* filep);

//This is entry point into all the unit test functions
NV_STATUS uvm_test_unit_test(
            UVM_TEST_UNIT_TEST_PARAMS* unit_test_params, struct file* filep);

NV_STATUS uvm_test_get_device_list(
            UVM_TEST_GET_DEVICE_LIST_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_register_channel(
            UVM_TEST_REGISTER_CHANNEL_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_unregister_channel(
            UVM_TEST_UNREGISTER_CHANNEL_PARAMS* params, struct file* filep);

NV_STATUS
uvmfull_test_set_page_table_update_policy(UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PARAMS *pParams,
                                      struct file *filp);

NV_STATUS uvm_debug_unmap(
            UVM_DEBUG_UNMAP_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_invalidate_tlb_entry(UvmVaspace *vaspace, NvU64 targetVA,
                                        UvmTargetVAMode targetVAMode,
                                        UvmTargetPdbMode targetPdb,
                                        UvmInvalidateTlbLevel tlbLevel,
                                        UvmInvalidateTlbMembarType membar,
                                        NvU32 gpcId, NvU32 clientId,
                                        UvmInvalidateTlbReplayType replayType,
                                        NvBool disableGpcInvalidate,
                                        UvmGpu *pGpu);

NV_STATUS uvm_test_invalidate_tlb(
                    UVM_TEST_INVALIDATE_TLB_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_get_va_pte_info(
        UVM_TEST_GET_VA_PTE_INFO_PARAMS* params, struct file* filep);

NV_STATUS uvmfull_test_merge_gpupde_pte(UvmVaspace *pVaspace,
                                        UvmGpuId     gpuid,
                                        NvU64         addr,
                                        NvU32         newPgSize
                                       );

NV_STATUS uvmfull_test_split_gpupde_pte(UvmVaspace *pVaspace,
                                        UvmGpuId     gpuid,
                                        NvU64         addr,
                                        NvU32         newPgSize,
                                        NvU32         splitFlags);

NV_STATUS uvm_test_merge_pages(
        UVM_TEST_MERGE_PAGES_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_split_pages(
        UVM_TEST_SPLIT_PAGES_PARAMS* params, struct file* filep);

NV_STATUS uvm_test_map_gpu_sysmem_pages(
        UVM_TEST_MAP_GPU_SYSMEM_PAGES_PARAMS *params, struct file* filep);

/*
 * 1. Allocate a page on specified GPU
 * 2. Write requested integer pattern in the page
 */
NV_STATUS uvm_test_create_zfod_page(
                UVM_TEST_CREATE_ZFOD_PAGE_PARAMS* params, struct file* filep);
/*
 * Enable the usage of ZFOD for a given gpuVaspace
 */
NV_STATUS uvm_test_configure_zfod_policy(
            UVM_TEST_CONFIGURE_ZFOD_POLICY_PARAMS* params, struct file* filep);
NV_STATUS
uvmfull_test_reset_fault_buffer_overflow(UVM_TEST_RESET_FAULT_BUFFER_OVERFLOW_PARAMS *pParams,
                                      struct file *filp);

NV_STATUS
uvmfull_test_configure_fault_handler(UVM_TEST_CONFIGURE_FAULT_HANDLER_PARAMS *pParams,
                                     struct file *filp);

NV_STATUS uvm_test_change_pte_mapping(
        UVM_TEST_CHANGE_PTE_MAPPING_PARAMS *params, struct file *filep);

NV_STATUS uvm_test_force_page_size(
        UVM_TEST_FORCE_PAGE_SIZE_PARAMS *params, struct file *filep);

NV_STATUS uvm_test_swap_gpu_pages(
        UVM_TEST_SWAP_GPU_PAGES_PARAMS *params, struct file *filep);

NV_STATUS uvm_test_explicit_va_migrate(
        UVM_TEST_EXPLICIT_VA_MIGRATE_PARAMS *params, struct file *filep);

#endif
