/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

// <unreviewed_code>

#include "uvm_common.h"
#include "uvm_full_test.h"
#include "uvm_full.h"
#include "uvmfull_test_ioctl.h"
#include "uvm_hashmap.h"
#include "uvm_full_va_trie.h"
#include "linux/random.h"

#if 0   // Needs to be updated for new design
// for i in [0 .. numKeys-1],
//      generates random keys[i] and inserts <keys[i], values[i]> in map
static NV_STATUS _insert_rand_values(UvmHashMap* map, unsigned bitOffset,
        unsigned *entryIdx, unsigned* values, unsigned numKeys, NvU64* keys)
{
    unsigned j;
    for (j = 0; j < numKeys; ++j)
    {
        get_random_bytes(&keys[j], 4);
        keys[j] <<= bitOffset;
    }

    for (j = 0; j < numKeys; ++j)
    {
        ++(*entryIdx);
        if(NV_OK != uvm_hashmap_insert(map, keys[j], (NvU64)&(values[j])))
        {
            UVM_ERR_PRINT("Hashmap insertion failed on %d th entry\n",
                          *entryIdx);
            return NV_ERR_INVALID_OPERATION;
        }
    }
    return NV_OK;
}
#endif

static unsigned _hashmap_block_insert_check(unsigned blockSize,
                                        unsigned numBlocks, unsigned bitOffset)
{
#if 0   // Needs to be updated for new design
    UvmHashMap map;
    unsigned *values;
    NvU64 *keys;
    NvU64 queriedValue;
    unsigned  i, j, keyIdx, entryIdx, _panic;

    memset(&map, 0, sizeof(map));
    uvm_hashmap_init(&map, bitOffset);

    keys   = (NvU64*)(vmalloc(blockSize * sizeof(NvU64) )); // keys are NvU64
    values = (unsigned *)(vmalloc(blockSize * sizeof(unsigned)));

    for (j = 0; j < blockSize; ++j)
        values[j] = j; // values are inconsequential.

    _panic = 1 ;

    for (i = 0; i < numBlocks; ++i)
    {
        entryIdx = i * blockSize;
        if(NV_OK != _insert_rand_values(&map, bitOffset, &entryIdx, values,
                                        blockSize, keys))
            goto cleanup;

        for (j = 0; j < blockSize*2; ++j)
        {
            ++entryIdx;
            get_random_bytes(&keyIdx, 4);
            if( NV_OK != uvm_hashmap_query(&map, keys[keyIdx], &queriedValue))
            {
                UVM_ERR_PRINT(
                       "Hashmap query failed on %d th entry, with key = %llu",
                       entryIdx, keys[keyIdx]);
                goto  cleanup;
            }

            if((unsigned)(queriedValue) != values[keyIdx])
            {
                UVM_ERR_PRINT(
                       "Hashmap returned incorrect value on query of %d entry",
                       entryIdx);
                goto  cleanup;
            }
        }
    }

    _panic = 0; // this stops the _panic message on cleanup

cleanup:
    vfree(keys);
    vfree(values);
    uvm_hashmap_destroy(&map);
    if(_panic == 1)
        UVM_PANIC();
#endif
    return NV_OK; // if we did not PANIC above, we are fine.
}

static unsigned _hashmap_block_delete_check(unsigned blockSize,
                    unsigned numBlocks, unsigned numDelete, unsigned bitOffset)
{
#if 0  // Needs to be updated for new design
    UvmHashMap map;
    NvU64 *keys;
    NvU64 queriedValue;
    unsigned  j, keyIdx, entryIdx, _panic, *values, *valuesVerify;
    unsigned numElems = blockSize * numBlocks;

    memset(&map, 0, sizeof(map));
    uvm_hashmap_init(&map, bitOffset);

    keys   = (NvU64*)(vmalloc(numElems * sizeof(NvU64))); // keys are NvU64
    values = (unsigned *)(vmalloc(numElems * sizeof(unsigned)));
    valuesVerify = (unsigned *)(vmalloc(numElems * sizeof(unsigned)));

    for (j = 0; j < blockSize; ++j)
        get_random_bytes(&(values[j]), 4), valuesVerify[j] = values[j];

    _panic = 1 ;
    entryIdx = 0;
    if (NV_OK != _insert_rand_values(&map, bitOffset, &entryIdx, values,
                                    numElems, keys))
        goto cleanup;

    entryIdx = 0;

    // Run length is proportional to ( n + n/(n-numDelete) )
    // {with n = numBlocks * blockSize}
    if(numDelete > numElems/4)
        UVM_PANIC_MSG(
             "Invalid test param: keys to delete > 1/4 of number inserted.\n");

    for (j = 0; j < numDelete; ++j)
    {
        get_random_bytes(&keyIdx , 4);
        keyIdx %= numElems;
        if(NV_OK != uvm_hashmap_remove(&map, keys[keyIdx]))
        {
            UVM_ERR_PRINT(
                    "Hashmap remove failed on %u th entry, with key = %llu",
                    entryIdx, keys[keyIdx]);
            goto  cleanup;
        }
        keys[keyIdx] = 0 ;
    }

    entryIdx = 0;
    // following checking loop will execute,
    // approx (0.25 * map->nItems / ((numElems-numDelete)/numElems))
    // So if there' are very large number of deletes, this will run longer
    do
    {
        get_random_bytes(&keyIdx , 4);
        keyIdx %= numElems;
        if( keys[keyIdx] == 0 ) // removed key
            continue;

        ++entryIdx;

        if(NV_OK != uvm_hashmap_query(&map, keys[keyIdx], &queriedValue))
        {
            UVM_ERR_PRINT(
                    "Hashmap query failed on %u th entry, with key = %llu",
                    entryIdx, keys[keyIdx]);
            goto  cleanup;
        }

        if((unsigned)(queriedValue) != valuesVerify[keyIdx])
        {
            UVM_ERR_PRINT(
                    "Hashmap returned incorrect value on query of %d entry",
                    entryIdx);
            goto  cleanup;
        }
    }while(entryIdx  < map.nItems/4);

    _panic = 0; // this stops the panic message on cleanup

cleanup:
    vfree(keys);
    vfree(values);
    vfree(valuesVerify);
    uvm_hashmap_destroy(&map);
    if(_panic)
        UVM_PANIC();
#endif

    return NV_OK; // if we did not PANIC above, we are fine.

}

static NV_STATUS unit_test_hashmap_test(HashmapTestInfo* info)
{
    switch (info->variety)
    {
        case Variety_BlockInsertAndCheck :
            return _hashmap_block_insert_check(info->blockSize, info->numBlocks,
                                                info->bitOffset);
        case Variety_BlockInsertDeleteCheck:
            return _hashmap_block_delete_check(info->blockSize, info->numBlocks,
                                          info->numDeleteKeys, info->bitOffset);
        default :
            UVM_PANIC_MSG("HashmapTestInfo test Variety is invalid");
    }

    return NV_OK;
}

static NV_STATUS _va_trie_generic_insert_check(NvU64 start, NvU64 len)
{
    NvU64 addr, end;
    UvmVaspace *vaspace = NULL; // dummy vaspace
    NV_STATUS status = NV_OK;
    UvmVatContext query = {0};
    Uvm2MbVaDesc *desc = NULL;

    vaspace = vmalloc(sizeof(*vaspace));
    if (!vaspace)
    {
        status = NV_ERR_NO_MEMORY;
        goto cleanup;
    }

    memset(vaspace, 0, sizeof(*vaspace));

    uvmfull_vat_init(&vaspace->vat);

    end = start + len;
    start = UVM_ALIGN_ADDR(start, PAGE_SIZE_2M);
    end = UVM_ALIGN_ADDR(end, PAGE_SIZE_2M) + PAGE_SIZE_2M - 1;

    // First touch all descriptors.
    for (addr = start; addr < end; addr += PAGE_SIZE_2M)
    {
        query.accessType = UvmVatAccess_GenericTouch;
        query.address = addr;
        query.vaspace = vaspace;
        desc = uvmfull_vat_acquire_leaf(&vaspace->vat, &query);
        if (!desc)
        {
            status = NV_ERR_GENERIC;
            goto cleanup;
        }

        if (desc->addr != addr)
            status = NV_ERR_GENERIC;

        uvmfull_vat_release_leaf(&vaspace->vat, &query);
        if (status != NV_OK)
            goto cleanup;
    }

    // Query all descriptors.
    for (addr = start; addr < end; addr += PAGE_SIZE_2M)
    {
        query.accessType = UvmVatAccess_GenericQuery;
        query.address = addr;
        query.vaspace = vaspace;
        desc = uvmfull_vat_acquire_leaf(&vaspace->vat, &query);
        if (!desc)
        {
            status = NV_ERR_GENERIC;
            goto cleanup;
        }

        if (desc->addr != addr)
            status = NV_ERR_GENERIC;

        uvmfull_vat_release_leaf(&vaspace->vat, &query);
        if (status != NV_OK)
            goto cleanup;
    }

cleanup:
    if (vaspace)
    {
        uvmfull_vat_destroy(&vaspace->vat);
        vfree(vaspace);
    }
    return status;
}

static NV_STATUS _va_trie_generic_insert_delete_check(NvU64 start, NvU64 length)
{
    return NV_ERR_GENERIC;
}

static NV_STATUS unit_test_va_trie_test(VaTrieTestInfo *info)
{
    switch (info->variety)
    {
        case Variety_genericInsertCheck :
            return _va_trie_generic_insert_check(info->start, info->length);
        case Variety_genericInsertDeleteCheck :
            return _va_trie_generic_insert_delete_check(
                                        info->start, info->length);
        default :
            return NV_ERR_INVALID_REQUEST;
    }
    return NV_OK;
}

NV_STATUS uvm_test_unit_test(UVM_TEST_UNIT_TEST_PARAMS* params,
                             struct file* filep)
{
    NV_STATUS status;
    switch(params->type)
    {
        case TestType_HashmapTest :
            status = unit_test_hashmap_test(
                    &params->closure.hashmapTestInfo);
            break;
        case TestType_VaTrieTest :
            status = unit_test_va_trie_test(
                    &params->closure.vaTrieTestInfo);
            break;
        default :
            status = NV_ERR_INVALID_REQUEST;
    }
    params->rmStatus = status;
    return NV_OK;
}

// </unreviewed_code>
