/*******************************************************************************
    Copyright (c) 2014-2015 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*******************************************************************************/

#ifndef __UVM_HASHMAP_H__
#define __UVM_HASHMAP_H__

#include "nvtypes.h"
#include "nvstatus.h"
#include "uvm_linux.h"

#ifndef NVIDIA_UVM_NEXT_CODE
#include "uvm_full_types.h"
#else
#include "uvm_next_types.h"
#endif

struct UvmHashEntry_tag
{
    NvU64                   key;
    struct hlist_node       node;
};

struct UvmHashMap_tag
{
    size_t                  nItems;      //Num of data items in the map
    size_t                  nSlots;
    struct hlist_head       *slots;      //Pointer to allocated array
};

// General design: The UvmHashMap implements a collision aware hash map.
// The map is an array of slots of size power of 2.
// Colliding entries are stored in a linked list hanging off the slot.

// Initializes the hashmap by allocating a slot array of size 2^nSlotsLog2
NV_STATUS uvm_hashmap_init(UvmHashMap *map, NvU8 nSlotsLog2);

// Deallocates the slot array. In the case when the slot array has entries,
// unlinks the entries followed by freeing the array.
void uvm_hashmap_destroy(UvmHashMap *map);

// Sets the key in the entry and inserts it into the map.Colliding entries are
// maintained in a list. Expects the client to not insert duplicate keys.
void uvm_hashmap_insert(UvmHashMap *map, UvmHashEntry *entry, NvU64 key);

// Unlinks the entry from the map. It is up to the user to deallocate the entry
// if needed.
void uvm_hashmap_remove(UvmHashMap *map, UvmHashEntry *entry);

// Given a key, returns the UvmHashEntry.
UvmHashEntry* uvm_hashmap_query(UvmHashMap *map, NvU64 key);

#endif  // __UVM_HASHMAP_H__
