/*******************************************************************************
    Copyright (c) 2013 NVidia Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*******************************************************************************/

#ifndef _UVMFULL_TEST_IOCTL_H_
#define _UVMFULL_TEST_IOCTL_H_

#include "uvmtypes.h"
#include "uvmfull_unit_test.h"
#include "uvm-debug.h"
#include "uvm_events.h"
#include "uvm_ioctl.h"
#include "uvm_fake_fault_info.h"

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_WIN32) || defined(_WIN64)
#define UVM_TEST_IOCTL_BASE(i)                     CTL_CODE(FILE_DEVICE_UNKNOWN, 0xD00+i,    \
                                                           METHOD_BUFFERED,                  \
                                                           FILE_READ_DATA | FILE_WRITE_DATA)
#else
#define UVM_IOCTL_BASE(i) i
#define UVM_TEST_IOCTL_BASE(i)                     UVM_IOCTL_BASE(200 + i)
#endif /*defined(_WIN32) || defined(_WIN64)*/

#define UVM_MAX_COUNTERS_PER_IOCTL_CALL 32

#define UVM_TEST_REGION_COMMIT                     UVM_TEST_IOCTL_BASE(2)
typedef struct {
    NvU64 requestedBase;
    NvLength length;
    UvmGpuUuid gpuUuid;
    NV_STATUS rmStatus;
} UVM_TEST_REGION_COMMIT_PARAMS;


#define UVM_TEST_SET_VA_STATE                      UVM_TEST_IOCTL_BASE(3)

#define UVM_TEST_SET_VA_STATE_PARAMS_APERTURE_VIDMEM     0
#define UVM_TEST_SET_VA_STATE_PARAMS_APERTURE_SYS_COH    1
#define UVM_TEST_SET_VA_STATE_PARAMS_APERTURE_SYS_NONCOH 2
typedef struct {
    NvU32 readwrite  : 1; // 0:readonly and 1: read-write
    NvU32 atomic     : 1;
    NvU32 valid      : 1;
    NvU32 allocate_bckgn: 1;
    NvU32 aperture; // 0: Vidmem, 1. Sysmem coherent, 2. Sysmem non-coherent

    NvU64 addr;
    NvU32 pageSize;
    NvU32 numPages;
    NvU32 invalidateLevel;

    UvmGpuUuid gpuUuid;

    NV_STATUS rmStatus;
} UVM_TEST_SET_VA_STATE_PARAMS;


#define UVM_TEST_SET_FAULT_POLICY                  UVM_TEST_IOCTL_BASE(4)
typedef struct  // See Uvm-software-test-plan-for-pascal-hw-validation.docx
{
    NvU32 read   : 1;   // read only
    NvU32 write  : 1;   // read-write
    NvU32 atomic : 1;

    NvU32 duplicate  : 1;
    NvU32 sysmemonly : 1;

    NvU32 pdeMigrate : 1; // restore on false

    NvU32 cancel :  1;

    NvU64 addr;
    NvU32 length;

    UvmGpuUuid gpuUuid; //set per gpu pdeMigrate Policy

    NV_STATUS rmStatus;


} UVM_TEST_SET_FAULT_POLICY_PARAMS;


#define UVM_TEST_REGISTER_DEVICE                UVM_TEST_IOCTL_BASE(5)
typedef  struct
{
    UvmGpuUuid gpuUuid;
    NvHandle   hClient;
    NvHandle   hVaspace;
    NV_STATUS rmStatus;
}UVM_TEST_REGISTER_DEVICE_PARAMS;

#define UVM_TEST_GET_DEVICE_LIST                UVM_TEST_IOCTL_BASE(6)
typedef  struct
{
#define UVM_TEST_DEVICES                        (64)
    struct
    {
        NvBool          bValidDevice;
        NvBool          bCanReplayFaults;
        UvmGpuUuid      gpuUuid;
    } device[UVM_TEST_DEVICES];
    NV_STATUS rmStatus;
} UVM_TEST_GET_DEVICE_LIST_PARAMS;

typedef enum
{
    TestType_None        = 0,
    TestType_HashmapTest = 1,
    TestType_VaTrieTest  = 2
}TestType;

#define UVM_TEST_UNIT_TEST                        UVM_TEST_IOCTL_BASE(7)
typedef struct
{
    TestType  type;
    UnitTestClosure closure; // any data, typically a struct * from uvmfull_unit_test.h for the test.
    NV_STATUS rmStatus;
}UVM_TEST_UNIT_TEST_PARAMS;

#define UVM_TEST_REGISTER_CHANNEL                UVM_TEST_IOCTL_BASE(13)
typedef  struct
{
    UvmGpuUuid gpuUuid;
    NvHandle   hClient;
    NvHandle   hChannel;
    NvHandle   hUvmChannelHandle;
    NV_STATUS rmStatus;
}UVM_TEST_REGISTER_CHANNEL_PARAMS;

#define UVM_TEST_UNREGISTER_CHANNEL              UVM_TEST_IOCTL_BASE(14)
typedef  struct
{
    UvmGpuUuid gpuUuid;
    NvHandle   hUvmChannelHandle;
    NV_STATUS rmStatus;
}UVM_TEST_UNREGISTER_CHANNEL_PARAMS;

#define UVM_DEBUG_UNMAP                 UVM_TEST_IOCTL_BASE(15)
typedef struct 
{
    unsigned long long base;              // IN
    unsigned long long length;            // IN
    NvBool             skipTlbInvalidate; // IN
    UvmProcessorUuid   processorUuid;     // IN
    NV_STATUS          rmStatus;          // OUT
}UVM_DEBUG_UNMAP_PARAMS;

#define UVM_TEST_FAKE_FAULT_INIT                UVM_TEST_IOCTL_BASE(16)
typedef struct
{
    UvmGpuUuid    * pGpuUuid;
    NvU32           bufferSize;
    NV_STATUS       status;
}UVM_TEST_FAKE_FAULT_INIT_PARAMS;

#define UVM_TEST_FAKE_FAULT_DEINIT              UVM_TEST_IOCTL_BASE(17)
typedef struct
{
    UvmGpuUuid    * pGpuUuid;
    NV_STATUS       status;
}UVM_TEST_FAKE_FAULT_DEINIT_PARAMS;

#define UVM_TEST_GENERATE_FAKE_FAULT            UVM_TEST_IOCTL_BASE(18)
typedef struct
{
    UvmGpuUuid          * pGpuUuid;
    NvHandle              hClient;
    NvHandle              hChannel;
    NvU64                 faultAddress;
    NvU64                 faultLength;
    NvU32                 pageSize;       // Page size of newly allocatable pages
    FakeFaultType         faultType;
    FakeFaultAccessType   faultAccessType;
    NV_STATUS     status;
}UVM_TEST_GENERATE_FAKE_FAULT_PARAMS;

#define UVM_TEST_SERVICE_FAKE_FAULT             UVM_TEST_IOCTL_BASE(19)
typedef struct
{
    UvmGpuUuid    * pGpuUuid;
    NV_STATUS       status;
}UVM_TEST_SERVICE_FAKE_FAULT_PARAMS;

#define UVM_TEST_FAKE_FAULT_SYNCH               UVM_TEST_IOCTL_BASE(20)
typedef struct
{
    UvmGpuUuid  * pGpuUuid;
    NvU64         faultAddress;
    NvU64         faultLength;
    NV_STATUS     status;
}UVM_TEST_FAKE_FAULT_SYNCH_PARAMS;

//
// UvmTestSetPageTableUpdatePolicy
//
#define UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY    UVM_TEST_IOCTL_BASE(21)

#define UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PAGE_SIZE_USE_DEFAULT           0
#define UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PAGE_SIZE_4KB                   (4*1024)
#define UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PAGE_SIZE_64KB                  (64*1024)
#define UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PAGE_SIZE_2MB                   (2*1024*1024)
typedef struct
{
    UvmGpuUuid         gpuUuid;                    // IN
    NvBool             bForcePteMemsets;           // IN
    NvU32              mappingSize;                // IN
    NvU32              faultServiceRange;          // IN
    NvBool             bForceMigration;            // IN
    NvBool             bForceReplayStartAckAll;    // IN
    NvBool             bForceP2PMigration;         // IN
    NV_STATUS          rmStatus;                   // OUT
} UVM_TEST_SET_PAGE_TABLE_UPDATE_POLICY_PARAMS;


#define UVM_TEST_INVALIDATE_TLB        UVM_TEST_IOCTL_BASE(22)
typedef struct{
    unsigned long long targetVa;                //IN
    UvmTargetVAMode targetVAMode;               //IN
    unsigned targetPdb;                         //IN
    UvmInvalidateTlbLevel tlbLevel;             //IN
    UvmInvalidateTlbMembarType membar;          //IN
    NvU32 gpcId;                                //IN
    NvU32 clientId;                             //IN
    UvmInvalidateTlbReplayType replayType;      //IN
    UvmGpuUuid gpuUuid;                         //IN
    NvBool disableGpcInvalidate;                //IN
    NV_STATUS rmStatus;                         // OUT
}UVM_TEST_INVALIDATE_TLB_PARAMS;


#define UVM_TEST_GET_VA_PTE_INFO        UVM_TEST_IOCTL_BASE(23)
typedef struct
{
    UvmGpuUuid              gpuUuid;             //IN
    unsigned long long      targetVa;            //IN
    NvU32                   pgSize;              //OUT
    NvU64                   valid:1;             //OUT
    NvU64                   readonly:1;          //OUT
    NvU64                   atomic:1;            //OUT
    //! vidmem,sysmem, sysmem-non-coherent
    NvU64                   aperture:2;          //OUT
    //! Relative gpu peer id
    NvU64                   peerid:3;            //OUT
    //! Volatile Pte
    NvU64                   vol:1;               //OUT
    //! The 4K aligned physical address in the pte
    NvU64                   pfn:35;              //OUT
    NvU64                   pfnvalid:1;          //OUT
    NvU32                   rmStatus;            //OUT
} UVM_TEST_GET_VA_PTE_INFO_PARAMS;


#define UVM_TEST_SPLIT_PAGES        UVM_TEST_IOCTL_BASE(24)
typedef struct
{
    UvmGpuUuid         gpuUuid;                //IN
    unsigned long long targetVa;               //IN
    unsigned long long length;                 //IN
    NvU32              newPgSize;              //IN
    NvBool             bForceFixedPg;          //IN
    NV_STATUS          rmStatus;               //OUT
} UVM_TEST_SPLIT_PAGES_PARAMS;


#define UVM_TEST_MAP_GPU_SYSMEM_PAGES    UVM_TEST_IOCTL_BASE(25)

typedef struct
{
    UvmGpuUuid         gpuUuid;                //IN
    unsigned long long targetVa;               //IN
    NvU32               gpuPgSize;              //IN
    NV_STATUS          rmStatus;               //OUT
} UVM_TEST_MAP_GPU_SYSMEM_PAGES_PARAMS;

#define UVM_TEST_CREATE_ZFOD_PAGE       UVM_TEST_IOCTL_BASE(26)
typedef struct {
    UvmGpuUuid gpuUuid;                         //IN
    NvU32      pattern;                         //IN
    NV_STATUS  rmStatus;                        //OUT
} UVM_TEST_CREATE_ZFOD_PAGE_PARAMS;

#define UVM_TEST_CONFIGURE_ZFOD_POLICY  UVM_TEST_IOCTL_BASE(27)
typedef struct {
    UvmGpuUuid gpuUuid;                         //IN
    NvBool     bZfodEnabled;                    //IN
    NV_STATUS rmStatus;                         //OUT
} UVM_TEST_CONFIGURE_ZFOD_POLICY_PARAMS;

//
// UvmTestResetFaultBufferOverflow
//
#define UVM_TEST_RESET_FAULT_BUFFER_OVERFLOW  UVM_TEST_IOCTL_BASE(28)
typedef struct
{
    UvmGpuUuid         gpuUuid;             // IN
    NV_STATUS          rmStatus;            // OUT
} UVM_TEST_RESET_FAULT_BUFFER_OVERFLOW_PARAMS;

//
// UvmTestConfigureFaultHandler
//
#define UVM_TEST_CONFIGURE_FAULT_HANDLER      UVM_TEST_IOCTL_BASE(29)
typedef struct
{
    UvmGpuUuid         gpuUuid;              // IN
    NvBool             bOnlyReplaysNoResolves; // IN
    NV_STATUS          rmStatus;             // OUT
} UVM_TEST_CONFIGURE_FAULT_HANDLER_PARAMS;

//
// UvmTestSwapGpuPages
//
#define UVM_TEST_SWAP_GPU_PAGES             UVM_TEST_IOCTL_BASE(30)
typedef struct
{
    UvmGpuUuid         gpuUuid;                 // IN
    unsigned           pageSize;                // IN
    unsigned long long vaA;                     // IN
    unsigned long long vaB;                     // IN
    NV_STATUS          rmStatus;                // OUT
} UVM_TEST_SWAP_GPU_PAGES_PARAMS;

//
// UvmTestMigrateGpuPages
//
#define UVM_TEST_EXPLICIT_VA_MIGRATE        UVM_TEST_IOCTL_BASE(31)
typedef struct
{
    unsigned long long va;                      // IN
    unsigned long long length;                  // IN
    NvBool             bMigrateToCpu;           // IN
    UvmGpuUuid         targetGpuUuid;           // IN
    NvBool             bRemapAfterMigration;    // IN
    NV_STATUS          rmStatus;                // OUT
} UVM_TEST_EXPLICIT_VA_MIGRATE_PARAMS;

//
// UvmTestChangePteMapping
//
#define UVM_TEST_CHANGE_PTE_MAPPING         UVM_TEST_IOCTL_BASE(32)

typedef enum
{
    UvmTestPteMapping_Invalid = 0,
    UvmTestPteMapping_ReadOnly,
    UvmTestPteMapping_ReadWrite,
    UvmTestPteMapping_ReadWriteAtomic,
    UvmTestPteMappingSize
} UVM_TEST_PTE_MAPPING;

typedef struct
{
    UvmGpuUuid              gpuUuid;                 // IN
    unsigned long long      va;                      // IN
    UVM_TEST_PTE_MAPPING    targetMapping;           // IN
    NV_STATUS               rmStatus;                // OUT
} UVM_TEST_CHANGE_PTE_MAPPING_PARAMS;

//
// UvmTestMigrateGpuPages
//
#define UVM_TEST_FORCE_PAGE_SIZE            UVM_TEST_IOCTL_BASE(33)

#define UVM_TEST_FORCE_PAGE_SIZE_4KB                   (4*1024)
#define UVM_TEST_FORCE_PAGE_SIZE_64KB                  (64*1024)
#define UVM_TEST_FORCE_PAGE_SIZE_2MB                   (2*1024*1024)

typedef struct
{
    UvmGpuUuid         gpuUuid;                 // IN
    unsigned long long va;                      // IN
    unsigned           pageSize;                // IN
    NV_STATUS          rmStatus;                // OUT
} UVM_TEST_FORCE_PAGE_SIZE_PARAMS;

#define UVM_TEST_MERGE_PAGES        UVM_TEST_IOCTL_BASE(34)
typedef struct
{
    UvmGpuUuid         gpuUuid;                //IN
    unsigned long long targetVa;               //IN
    NvU32              newPgSize;              //IN
    NV_STATUS          rmStatus;               //OUT
} UVM_TEST_MERGE_PAGES_PARAMS;

#ifdef __cplusplus
}
#endif


#endif // _UVMFULL_TEST_IOCTL_H_
