/*******************************************************************************
Copyright (c) 2013-2014 NVidia Corporation

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*******************************************************************************/

#ifndef __UVMFULL_UNIT_TEST_H__
#define __UVMFULL_UNIT_TEST_H__

typedef unsigned long long UT_STATUS; // unit test status

typedef enum
{
    Variety_None               ,
    Variety_BlockInsertAndCheck,
    Variety_BlockInsertDeleteCheck
} HashmapTest_Variety;

typedef struct
{
    HashmapTest_Variety variety;  // Type of testing to be done
    unsigned blockSize;           // Batched insertion: size of batch
    unsigned numBlocks;           // number of such blocks
    unsigned numDeleteKeys;       // Number of deletions to perform
    unsigned bitOffset;           // Start of the 32 bits that contains the entroy for hashing key
} HashmapTestInfo;

typedef enum
{
    // Insert the descs and check for presence
    Variety_genericInsertCheck,
    // Insert descs, check presence, remove, check absence
    Variety_genericInsertDeleteCheck,
} VaTrieTest_Variety;

typedef struct
{
    unsigned long start;
    unsigned long length;
    VaTrieTest_Variety variety;
} VaTrieTestInfo;

typedef union
{
    HashmapTestInfo hashmapTestInfo;
    VaTrieTestInfo vaTrieTestInfo;
} UnitTestClosure;

typedef enum
{
    UvmInvalidateTlbReplayType_none,
    UvmInvalidateTlbReplayType_start,
    UvmInvalidateTlbReplayType_start_ack_all,
    UvmInvalidateTlbReplayType_cancel_targeted,
    UvmInvalidateTlbReplayType_cancel_global,
    UvmInvalidateTlbReplayType_unsupported,
} UvmInvalidateTlbReplayType;


typedef enum
{
    UvmInvalidateTlbMemBar_none,
    UvmInvalidateTlbMemBar_sys,
    UvmInvalidateTlbMemBar_local,
}UvmInvalidateTlbMembarType;

typedef enum
{
  UvmInvalidateTlbLevel_all,
  UvmInvalidateTlbLevel_pte,
  UvmInvalidateTlbLevel_pl0,
  UvmInvalidateTlbLevel_pl1,
  UvmInvalidateTlbLevel_pl2,
  UvmInvalidateTlbLevel_pl3
} UvmInvalidateTlbLevel;

typedef enum
{
  UvmTargetPdbMode_All,
  UvmTargetPdbMode_One
} UvmTargetPdbMode;

typedef enum
{
    UvmTargetVA_All,
    UvmTargetVA_Targeted,
} UvmTargetVAMode;

#endif // __UVMFULL_UNIT_TEST_H__
